@@ -1,6 +1,6 @@
-##' Extract the fixed-effects estimates
+##' Extract Fixed Effects
 ##'
-##' Extract the estimates of the fixed-effects parameters from a fitted model.
+##' Extract fixed effects from a fitted \code{glmmTMB} model.
 ##' @name fixef
 ##' @title Extract fixed-effects estimates
 ##' @aliases fixef fixef.glmmTMB
@@ -9,20 +9,30 @@
 ##' be extracted.
 ##' @param \dots optional additional arguments. Currently none are used in any
 ##' methods.
-##' @return a named, numeric vector of fixed-effects estimates.
+##' @return an object of class \code{fixef.glmmTMB} comprising a list of components (\code{cond}, \code{zi}, \code{disp}), each containing a (possibly zero-length) numeric vector of coefficients 
 ##' @keywords models
+##' @details The print method for \code{fixef.glmmTMB} object \emph{only displays non-trivial components}: in particular, the dispersion parameter estimate is not printed for models with a single (intercept) dispersion parameter (see examples)
 ##' @examples
 ##' data(sleepstudy, package = "lme4")
-##' fixef(glmmTMB(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy))
+##' fm1 <- glmmTMB(Reaction ~ Days, sleepstudy)
+##' (f1 <- fixef(fm1))
+##' f1$cond
+##' ## show full coefficients, including dispersion parameter
+##' unlist(f1)
+##' print.default(f1)
 ##' @importFrom nlme fixef
 ##' @export fixef
 ##' @export
 fixef.glmmTMB <- function(object, ...) {
+  getXnm <- function(suffix) {
+      nm <- paste0("X",suffix)
+      return(colnames(getME(object, nm)))
+  }
   pl <- object$obj$env$parList(object$fit$par, object$fit$parfull)
-  structure(list(cond = setNames(pl$beta,   colnames(getME(object, "X"))),
-                 zi    = setNames(pl$betazi, colnames(getME(object, "Xzi"))),
-                 disp = setNames(pl$betad, colnames(getME(object, "Xd")))),
-            class =  "fixef.glmmTMB")
+  structure(list(cond = setNames(pl$beta,   getXnm("")),
+                 zi   = setNames(pl$betazi, getXnm("zi")),
+                 disp = setNames(pl$betad,  getXnm("d"))),
+            class = "fixef.glmmTMB")
 }
 
 ## general purpose matching between component names and printable names
@@ -30,21 +40,33 @@ cNames <- list(cond = "Conditional model",
                zi = "Zero-inflation model",
                disp = "Dispersion model")
 
+## check identity without worrying about environments etc.
+ident <- function(x,target) isTRUE(all.equal(x,target))
+
+formComp <- function(object,type="dispformula",target) {
+    ident(object$modelInfo$allForm[[type]],target) ||
+        ident(object$call[[type]],target)
+}
 ## FIXME: this is a bit ugly. On the other hand, a single-parameter
 ## dispersion model without a (... ?)
+
 trivialDisp <- function(object) {
-    ## This version works on summary object or fitted model object
-    ## FIXME: is there a better way to strip the environment before
-    ## comparing?
-    identical(deparse(object$call$dispformula),"~1")
+    formComp(object,"dispformula",~1)
 }
+
+zeroDisp <- function(object) {
+    formComp(object,"dispformula",~0)
+}
+
+## no roxygen for now ...
+# @param xnm vector of fixed-effect parameter names (e.g. names(fixef(m)$disp))
+# @param nm name of component (e.g. "disp")
 trivialFixef <- function(xnm,nm) {
     length(xnm)==0 ||
         (nm %in% c('d','disp') && identical(xnm,'(Intercept)'))
     ## FIXME: inconsistent tagging; should change 'Xd' to 'Xdisp'?
 }
 
-
 ##' @method print fixef.glmmTMB
 ##' @export
 print.fixef.glmmTMB <- function(x, digits = max(3, getOption("digits") - 3), ...)
@@ -60,14 +82,13 @@ print.fixef.glmmTMB <- function(x, digits = max(3, getOption("digits") - 3), ...
 
 ##' Extract Random Effects
 ##'
-##' Generic function to extract random effects from \code{glmmTMB} models, both
+##' Extract random effects from a fitted \code{glmmTMB} model, both
 ##' for the conditional model and zero inflation.
 ##'
 ##' @param object a \code{glmmTMB} model.
-##' @param condVar include conditional variances in result?
-##' @param ... some methods for this generic function require additional
-##'   arguments.
-##'
+##' @param condVar whether to include conditional variances in result.
+##' @param \dots some methods for this generic function require additional
+##'   arguments (they are unused here and will trigger an error)
 ##' @return
 ##' \itemize{
 ##' \item For \code{ranef}, an object of class \code{ranef.glmmTMB} with two components:
@@ -82,32 +103,32 @@ print.fixef.glmmTMB <- function(x, digits = max(3, getOption("digits") - 3), ...
 ##' random effects terms) will have associated \code{condVar} attributes
 ##' giving the conditional variances of the random effects values.
 ##' These are in the form of three-dimensional arrays: see
-##' \code{\link{ranef.merMod}} for details (the only difference between
-##' the packages is that the attributes are called \sQuote{"postVar"}
-##' in \pkg{lme4}, vs. \sQuote{"condVar"} in \pkg{glmmTMB}.
+##' \code{\link{ranef.merMod}} for details. The only difference between
+##' the packages is that the attributes are called \sQuote{postVar}
+##' in \pkg{lme4}, vs. \sQuote{condVar} in \pkg{glmmTMB}.
 ##' \item For \code{coef.glmmTMB}: a similar list, but containing
-##' the overall coefficient value for each level (i.e., the sum of
+##' the overall coefficient value for each level, i.e., the sum of
 ##' the fixed effect estimate and the random effect value for that
-##' level). \emph{Conditional variances are not yet available as
-##' an option for \code{coef.glmmTMB}.}
+##' level. \emph{Conditional variances are not yet available as
+##' an option for} \code{coef.glmmTMB}.
 ##' \item For \code{as.data.frame}: a data frame with components
 ##' \describe{
 ##' \item{component}{part of the model to which the random effects apply (conditional or zero-inflation)}
 ##' \item{grpvar}{grouping variable}
-##' \item{term}{random-effects term (e.g., intercept or slope}
+##' \item{term}{random-effects term (e.g., intercept or slope)}
 ##' \item{grp}{group, or level of the grouping variable}
 ##' \item{condval}{value of the conditional mode}
 ##' \item{condsd}{conditional standard deviation}
 ##' }
 ##' }
-##' 
+##'
 ##' @note When a model has no zero inflation, the
-##'   the \code{ranef} and \code{coef} print methods simplify the
-##'   structure shown, by default. To show the full list structure, use
-##'   \code{print(ranef(model),simplify=FALSE)} (or the analogous
-##' code for \code{coef}).
+##' \code{ranef} and \code{coef} print methods simplify the
+##' structure shown, by default. To show the full list structure, use
+##' \code{print(ranef(model),simplify=FALSE)} or the analogous
+##' code for \code{coef}.
 ##' In all cases, the full list structure is used to access
-##'   the data frames (see example).
+##' the data frames, see example.
 ##'
 ##' @seealso \code{\link{fixef.glmmTMB}}.
 ##'
@@ -126,6 +147,7 @@ print.fixef.glmmTMB <- function(x, digits = max(3, getOption("digits") - 3), ...
 ##' @export ranef
 ##' @export
 ranef.glmmTMB <- function(object, condVar=TRUE, ...) {
+  check_dots(...)
   ## The arrange() function converts a vector of random effects to a list of
   ## data frames, in the same way as lme4 does.
   ## FIXME: add condVar, make sure format matches lme4
@@ -185,7 +207,7 @@ ranef.glmmTMB <- function(object, condVar=TRUE, ...) {
   } ## arrange()
 
   pl <- getParList(object)  ## see VarCorr.R
-  if (condVar && has.random(object))  {
+  if (condVar && hasRandom(object))  {
       ss <- summary(object$sdr,"random")
       sdl <- list(b=ss[rownames(ss)=="b","Std. Error"],
                   bzi=ss[rownames(ss)=="bzi","Std. Error"])
@@ -243,12 +265,13 @@ getME.glmmTMB <- function(object,
   oo.env <- object$obj$env
   ### Start of the switch
   allpars <- oo.env$parList(object$fit$par, object$fit$parfull)
+  isSparse <- function(component) { if (is.null(om <- object$modelInfo$sparseX)) FALSE else om[[component]] }
   switch(name,
-         "X"     = oo.env$data$X,
-         "Xzi"   = oo.env$data$Xzi,
+         "X"     = if (!isSparse("cond")) oo.env$data$X else oo.env$data$XS,
+         "Xzi"   = if (!isSparse("zi")) oo.env$data$Xzi else oo.env$data$XziS,
          "Z"     = oo.env$data$Z,
          "Zzi"   = oo.env$data$Zzi,
-         "Xd"    = oo.env$data$Xd,
+         "Xd"    = if (!isSparse("disp")) oo.env$data$Xd else oo.env$data$XdS,
          "theta" = allpars$theta ,
          "beta"  = unlist(allpars[c("beta","betazi","betad")]),
          "..foo.." = # placeholder!
@@ -301,16 +324,16 @@ df.residual.glmmTMB <- function(object, ...) {
 ##'
 ##' @param object a \dQuote{glmmTMB} fit
 ##' @param full return a full variance-covariance matrix?
+##' @param include_mapped include mapped variables? (these will be given variances and covariances of NA)
 ##' @param \dots ignored, for method compatibility
 ##' @return By default (\code{full==FALSE}), a list of separate variance-covariance matrices for each model component (conditional, zero-inflation, dispersion).  If \code{full==TRUE}, a single square variance-covariance matrix for \emph{all} top-level model parameters (conditional, dispersion, and variance-covariance parameters)
 ##' @importFrom TMB MakeADFun sdreport
 ##' @importFrom stats vcov
 ##' @export
-vcov.glmmTMB <- function(object, full=FALSE, ...) {
-  if (is.null(REML <- object$modelInfo$REML)) {
-     ## let vcov work with old (pre-REML option) stored objects
-     REML <- FALSE
-  }
+vcov.glmmTMB <- function(object, full=FALSE, include_mapped=FALSE, ...) {
+  ## don't check_dots, car::Anova tries to pass 'complete'
+  ## check_dots(...)  
+  REML <- isREML(object)
   if(is.null(sdr <- object$sdr)) {
     warning("Calculating sdreport. Use se=TRUE in glmmTMB to avoid repetitive calculation of sdreport")
     sdr <- sdreport(object$obj, getJointPrecision=REML)
@@ -335,7 +358,8 @@ vcov.glmmTMB <- function(object, full=FALSE, ...) {
       X <- getME(object,paste0("X",tag))
       if (trivialFixef(nn <- colnames(X),tag)
           ## if 'full', keep disp even if trivial, if used by family
-          && !(full && tag =="d" && usesDispersion(family(object)$family))) {
+          && !(full && tag =="d" &&
+               (usesDispersion(family(object)$family) && !zeroDisp(object)))) {
           return(character(0))
       }
       return(paste(tag,nn,sep="~"))
@@ -360,26 +384,57 @@ vcov.glmmTMB <- function(object, full=FALSE, ...) {
           if (length(nn)==0) return(nn)
           return(paste("theta",gsub(" ","",nn),sep="_"))
       }
-      nameList <- c(nameList,list(reNames("cond"),reNames("zi")))
+      ## nameList for estimated variables;
+      nameList <- c(nameList,list(theta=reNames("cond"),thetazi=reNames("zi")))
+  }
 
-      colnames(covF) <- rownames(covF) <- unlist(nameList)
+
+  ## drop NA-mapped variables
+
+  ## for matching map names vs nameList components ...
+  par_components <- c("beta","betazi","betad","theta","thetazi","thetaf")
+
+  fullNameList <- nameList
+  map <- object$obj$env$map
+  if (length(map)>0) {
+        ## fullNameList for all variables, including mapped vars
+      ## (nameList will get reduced shortly)
+      for (m in seq_along(map)) {
+          if (length(NAmap <- which(is.na(map[[m]])))>0) {
+              w <- match(names(map)[m],par_components) ##
+              if (length(nameList)>=w) { ## may not exist if !full
+                  nameList[[w]] <- nameList[[w]][-NAmap]
+              }
+          }
+      }
+  }
+
+  if (full) {
+        colnames(covF) <- rownames(covF) <- unlist(nameList)
       res <- covF        ## return just a matrix in this case
   } else {
-      splitMat <- function(x) {
-          ss <- split(seq_along(colnames(x)),
-                      colnames(x))
-          lapply(ss,function(z) x[z,z,drop=FALSE])
-      }
-      covList <- splitMat(covF)
-      names(covList) <-
-          names(cNames)[match(names(covList),c("beta","betazi","betad"))]
-      for (nm in names(covList)) {
-          if (length(xnms <- nameList[[nm]])==0) {
-              covList[[nm]] <- NULL
+      ## extract block-diagonal matrix
+      ss <- split(seq_along(colnames(covF)), colnames(covF))
+      covList <- vector("list",3)
+      names(covList) <- names(cNames) ## component names
+      parnms <- c("beta","betazi", "betad")     ## parameter names 
+      for (i in seq_along(covList)) {
+          nm <- parnms[[i]]
+          m <- covF[ss[[nm]],ss[[nm]], drop=FALSE]
+          cnm <- names(covList)[[i]]
+          xnms <- nameList[[cnm]]
+          if (!include_mapped || length(map)==0) {
+              dimnames(m) <- list(xnms,xnms)
+          } else {
+              fnm <- fullNameList[[cnm]]
+              mm <- matrix(NA_real_,length(fnm),length(fnm),
+                           dimnames=list(fnm,fnm))
+              mm[nameList[[cnm]],nameList[[cnm]]] <- m
+              m <- mm
           }
-          else dimnames(covList[[nm]]) <- list(xnms,xnms)
+          covList[[i]] <- m
       }
-      res <- covList
+      res <- covList[lengths(covList)>0]
       ##  FIXME: should vcov always return a three-element list
       ## (with NULL values for trivial models)?
       class(res) <- c("vcov.glmmTMB","matrix")
@@ -478,7 +533,7 @@ printDispersion <- function(ff,s) {
             sname <- "sigma^2"
             sval <- s^2
         } else {
-            dname <- "Overdispersion parameter"
+            dname <- "Dispersion parameter"
             sname <- ""
             sval <- s
         }
@@ -490,7 +545,7 @@ printDispersion <- function(ff,s) {
 }
 
 .tweedie_power <- function(object) {
-    unname(plogis(object$fit$par["thetaf"]) + 1)
+    unname(plogis(get_pars(object)["thetaf"]) + 1)
 }
 
 ## Print family specific parameters
@@ -571,7 +626,8 @@ residuals.glmmTMB <- function(object, type=c("response", "pearson"), ...) {
     if(type=="pearson" &((object$call$ziformula != ~0)|(object$call$dispformula != ~1))) {
         stop("pearson residuals are not implemented for models with zero-inflation or variable dispersion")
     }
-    mr <- model.response(object$frame)
+    na.act <- attr(object$frame,"na.action")
+    mr <- napredict(na.act,model.response(object$frame))
     wts <- model.weights(model.frame(object))
     ## binomial model specified as (success,failure)
     if (!is.null(dim(mr))) {
@@ -609,6 +665,13 @@ residuals.glmmTMB <- function(object, type=c("response", "pearson"), ...) {
 ## function, i.e. a function of 'fit$par' and/or 'fit$parfull'.
 ## Examples: 'sigma.glmmTMB' and some parts of 'VarCorr.glmmTMB'.
 
+## NOT roxygen (##') comments, as we don't want to trigger Rd file creation
+## @param object fitted model
+## @param f function
+## @param reduce
+## @param name.prepend
+## @param estimate
+
 ##' @importFrom stats qchisq
 .CI_univariate_monotone <- function(object, f, reduce=NULL,
                                     level=0.95,
@@ -635,7 +698,7 @@ residuals.glmmTMB <- function(object, type=c("response", "pearson"), ...) {
     nm <- names(ans)
     tmp <- cbind(ans$lower, ans$upper)
     if (is.null(tmp) || nrow(tmp) == 0L) return (NULL)
-    sort2 <- function(x) if(any(is.nan(x))) x * NaN else sort(x)
+    sort2 <- function(x) if(any(is.na(x))) x*NA else sort(x)
     ans <- cbind( t( apply(tmp, 1, sort2) ) , ans$Estimate )
     colnames(ans) <- nm
     if (!is.null(name.prepend))
@@ -657,26 +720,40 @@ format.perc <- function (probs, digits) {
 ##' @details
 ##' Available methods are
 ##' \describe{
-##' \item{wald}{These intervals are based on the standard errors
+##' \item{"wald"}{These intervals are based on the standard errors
 ##' calculated for parameters on the scale
 ##' of their internal parameterization depending on the family. Derived
 ##' quantities such as standard deviation parameters and dispersion
-##' parameters are backtransformed. It follows that confidence
-##' intervals for these derived quantities are asymmetric.}
-##' \item{profile}{This method computes a likelihood profile
+##' parameters are back-transformed. It follows that confidence
+##' intervals for these derived quantities are typically asymmetric.}
+##' \item{"profile"}{This method computes a likelihood profile
 ##' for the specified parameter(s) using \code{profile.glmmTMB};
 ##' fits a spline function to each half of the profile; and
 ##' inverts the function to find the specified confidence interval.}
-##' \item{uniroot}{This method uses the \code{\link{uniroot}}
+##' \item{"uniroot"}{This method uses the \code{\link{uniroot}}
 ##' function to find critical values of one-dimensional profile
 ##' functions for each specified parameter.}
 ##' }
-##'
+##' At present, "wald" returns confidence intervals for variance
+##' parameters on the standard deviation/correlation scale, while
+##' "profile" and "uniroot" report them on the underlying ("theta")
+##' scale: for each random effect, the first set of parameter values
+##' are standard deviations on the log scale, while remaining parameters
+##' represent correlations on the scaled Cholesky scale (see the
+##' 
+##' 
 ##' @importFrom stats qnorm confint
 ##' @export
 ##' @param object \code{glmmTMB} fitted object.
-##' @param parm Specification of a parameter subset \emph{after}
-##'     \code{component} subset has been applied.
+##' @param parm which parameters to profile, specified
+#' \itemize{
+#' \item by index (position) [\emph{after} component selection for \code{confint}, if any]
+#' \item by name (matching the row/column names of \code{vcov(object,full=TRUE)})
+#' \item as \code{"theta_"} (random-effects variance-covariance parameters), \code{"beta_"} (conditional and zero-inflation parameters), or \code{"disp_"} or \code{"sigma"} (dispersion parameters)
+#' }
+#'  Parameter indexing by number may give unusual results when
+#'  some parameters have been fixed using the \code{map} argument:
+#'  please report surprises to the package maintainers.
 ##' @param level Confidence level.
 ##' @param method 'wald', 'profile', or 'uniroot': see Details
 ##' function)
@@ -686,6 +763,7 @@ format.perc <- function (probs, digits) {
 ##' @param parallel method (if any) for parallel computation
 ##' @param ncpus number of CPUs/cores to use for parallel computation
 ##' @param cl cluster to use for parallel computation
+##' @param full CIs for all parameters (including dispersion) ?
 ##' @param ... arguments may be passed to \code{\link{profile.merMod}} or
 ##' \code{\link[TMB]{tmbroot}}
 ##' @examples
@@ -696,7 +774,7 @@ format.perc <- function (probs, digits) {
 ##' confint(model)  ## Wald/delta-method CIs
 ##' confint(model,parm="theta_")  ## Wald/delta-method CIs
 ##' confint(model,parm=1,method="profile")
-confint.glmmTMB <- function (object, parm, level = 0.95,
+confint.glmmTMB <- function (object, parm = NULL, level = 0.95,
                              method=c("wald",
                                       "Wald",
                                       "profile",
@@ -706,8 +784,8 @@ confint.glmmTMB <- function (object, parm, level = 0.95,
                              parallel = c("no", "multicore", "snow"),
                              ncpus = getOption("profile.ncpus", 1L),
                              cl = NULL,
-                             ...)
-{
+                             full = FALSE,
+                             ...) {
     method <- tolower(match.arg(method))
     if (method=="wald") {
         dots <- list(...)
@@ -728,75 +806,90 @@ confint.glmmTMB <- function (object, parm, level = 0.95,
     pct <- format.perc(a, 3)
     fac <- qnorm(a)
     estimate <- as.logical(estimate)
-    ci <- matrix(NA, 0, 2 + estimate,
+    ci <- matrix(NA, nrow=0, ncol=2 + estimate,
                  dimnames=list(NULL,
-                               c(pct, "Estimate")
-                               [c(TRUE, TRUE, estimate)] ))
-    pnm <- names(object$obj$par)
-    pvec <- seq_along(pnm)
-    if (!missing(parm)) {
-            ## FIXME: DRY/refactor with confint.profile
-            ## FIXME: beta_ not well defined; sigma parameters not
-            ## distinguishable (all called "sigma")
-            ## for non-trivial dispersion model
-            theta_parms <- which(pnm=="theta")
-            ## if non-trivial disp, keep disp parms for "beta_"
-            disp_parms <- if (!trivialDisp(object)) numeric(0) else grep("^sigma",rownames(ci))
-            if (identical(parm,"theta_")) {
-                parm <- theta_parms
-            } else if (identical(parm,"beta_")) {
-                parm <- seq(nrow(ci))[setdiff(pvec,c(theta_parms,disp_parms))]
-            }
-    } else {
-        parm <- pvec
+                               if (!estimate) pct else c(pct, "Estimate")))
+    
+    if (!is.null(parm) || method!="wald") {
+        parm <- getParms(parm, object, full)
     }
-    if (method=="wald") {
-        for (component in c("cond", "zi") ) {
-            if (components.has(component)) {
-                cf <- unlist(fixef(object)[component])
-                vv <- vcov(object)[component]
-                ss <- unlist(lapply(vv,diag))
-                ses <- sqrt(ss)
-                ci.tmp <- cf + ses %o% fac
-                if (estimate) ci.tmp <- cbind(ci.tmp, cf)
-                ci <- rbind(ci, ci.tmp)
-                ## VarCorr -> stddev
-                cfun <- function(x) {
-                    ss <- attr(x, "stddev")
-                    names(ss) <- paste(component,"Std.Dev",names(ss),sep=".")
-                    cc <- attr(x,"correlation")
-                    if (length(cc)>1) {
-                        nn <- outer(colnames(cc),rownames(cc),paste,sep=".")
-                        cc <- cc[lower.tri(cc)]
-                        nn <- paste(component,"Cor",nn[lower.tri(nn)],sep=".")
-                        names(cc) <- nn
-                        ss <- c(ss,cc)
-                    }
-                    return(ss)
-                }
-                reduce <- function(VC) sapply(VC[[component]], cfun)
-                ci.sd <- .CI_univariate_monotone(object,
-                                                 VarCorr,
-                                                 reduce = reduce,
-                                                 level = level,
-                                                 ## name.prepend=paste(component,
-                                                 ## "Std.Dev.",
-                                                 ## sep="."),
-                                                 estimate = estimate)
-                ci <- rbind(ci, ci.sd)
-            }
+
+    wald_comp <- function(component) {
+        vv <- vcov(object)[[component]]
+        cf <- fixef(object)[[component]]
+        ## strip tag (only really necessary for zi~, d~)
+        tag <- if (component=="disp") "d" else component
+        nn <- gsub(paste0(tag,"~"),"",colnames(vv))
+        ## vcov only includes estimated (not mapped/fixed)
+        ##  fixed-effect parameters
+        cf <- cf[nn]
+        ss <- diag(vv)
+        ## using [[-extraction; need to add component name explicitly
+        ci.tmp <- NULL
+        if (length(cf)>0) {
+            names(cf) <- names(ss) <-
+                paste(component, names(cf), sep=".")
+            ses <- sqrt(ss)
+            ci.tmp <- cf + ses %o% fac
+            if (estimate) ci.tmp <- cbind(ci.tmp, cf)
         }
+        return(ci.tmp)
+    }
+
+    wald_ci_comp <- function(component) {
+        ## VarCorr -> stddev
+        cfun <- function(x) {
+            ss <- attr(x, "stddev")
+            names(ss) <- paste(component,"Std.Dev",names(ss),sep=".")
+            cc <- attr(x,"correlation")
+            if (length(cc)>1) {
+                nn <- outer(colnames(cc),rownames(cc),paste,sep=".")
+                cc <- cc[lower.tri(cc)]
+                nn <- paste(component,"Cor",nn[lower.tri(nn)],sep=".")
+                names(cc) <- nn
+                ss <- c(ss,cc)
+            }
+            return(ss)
+        }
+        reduce <- function(VC) sapply(VC[[component]], cfun)
+        ci.sd <- .CI_univariate_monotone(object,
+                                         VarCorr,
+                                         reduce = reduce,
+                                         level = level,
+                                         estimate = estimate)
+        ## would consider excluding mapped parameters here
+        ## (works automatically for fixed effects via vcov)
+        ## but tough because of theta <-> sd/corr mapping;
+        ## instead, eliminate rows below where lowerCI==upperCI
+        return(ci.sd)
+    }
+    
+    if (method=="wald") {
+        map <- object$modelInfo$map
+        for (component in c("cond", "zi") ) {
+            if (components.has(component) &&
+                length(fixef(object)[[component]])>0) {
+                ## variance and estimates
+                ci <- rbind(ci, wald_comp(component))
+            }
+        } ## cond and zi components
         if (components.has("other")) {
             ## sigma
+            component <- "disp"
             ff <- object$modelInfo$family$family
             if (usesDispersion(ff)) {
-                ci.sigma <- .CI_univariate_monotone(object,
+                if (!trivialDisp(object) &&
+                    length(fixef(object)[[component]])>0) {
+                    ci <- rbind(ci, wald_comp(component))
+                } else {
+                    ci.sigma <- .CI_univariate_monotone(object,
                                                     sigma,
                                                     reduce = NULL,
                                                     level=level,
                                                     name.prepend="sigma",
                                                     estimate = estimate)
-                ci <- rbind(ci, ci.sigma)
+                    ci <- rbind(ci, ci.sigma)
+                }
             }
             ## Tweedie power
             if (ff == "tweedie") {
@@ -809,10 +902,33 @@ confint.glmmTMB <- function (object, parm, level = 0.95,
                 ci <- rbind(ci, ci.power)
             } ## tweedie
         }  ## model has 'other' component
+        ## NOW add 'theta' components (match order of params in vcov-full)
+        ## FIXME: better to have more robust ordering
+        for (component in c("cond", "zi") ) {
+            if (components.has(component) &&
+                length(ranef(object)[[component]])>0) {
+                ci <- rbind(ci, wald_ci_comp(component))
+            }
+        }
+            
         ## Take subset
-        ci <- ci[parm, , drop=FALSE]
+        
+        ## drop mapped values (where lower == upper)
+        ci <- ci[ci[,2]!=ci[,1], , drop=FALSE]
+        
+        ## now get selected parameters
+        if (!is.null(parm)) {
+            ci <- ci[parm, , drop=FALSE]
+        } else {
+            ## drop residual std dev/trivial dispersion parameter
+            if (!full) {
+                ci <- ci[rownames(ci)!="sigma",, drop=FALSE]
+            }
+        }
+
         ## end Wald method
     } else if (method=="uniroot") {
+        if (isREML(object)) stop("can't compute profiles for REML models at the moment (sorry)")
         ## FIXME: allow greater flexibility in specifying different
         ##  ranges, etc. for different parameters
         plist <- parallel_default(parallel,ncpus)
@@ -859,16 +975,27 @@ confint.glmmTMB <- function (object, parm, level = 0.95,
                       ...)
         ci <- confint(pp)
     }
+    ## if only conditional, strip component prefix
+    if (all(substr(rownames(ci),1,5)=="cond.")) {
+        rownames(ci) <- sub("^cond\\.","",rownames(ci))
+    }
     return(ci)
 }
 
+##' @rdname glmmTMB_methods
+##' @param x a fitted \code{glmmTMB} object
 ##' @export
-## FIXME: establish separate 'terms' components for
-##   each model component (conditional, random, zero-inflation, dispersion ...)
+##  modified because e.g. "disp" component didn't get a $reTrms
+##  component (updated fitTMB to save a separate "terms" component)
 terms.glmmTMB <- function(x, component="cond", part="fixed", ...) {
     if (part != "fixed") stop("only fixed terms currently available")
-    return(x$modelInfo$reTrms[[component]]$terms[[part]])
-    ## terms(x$frame)
+    if ("terms" %in% names(x$modelInfo)) {
+        tt <- x$modelInfo$terms[[component]]
+    } else {
+        ## allow back-compatibility
+        tt <- x$modelInfo$reTrms[[component]]$terms
+    }
+    return(tt[[part]])
 }
 
 ##' @export
@@ -879,16 +1006,42 @@ extractAIC.glmmTMB <- function(fit, scale, k = 2, ...) {
 }
 
 ## deparse(.) returning \bold{one} string
-## copied from lme4/R/utilities.R
+## previously safeDeparse; 
 ## Protects against the possibility that results from deparse() will be
 ##       split after 'width.cutoff' (by default 60, maximally 500)
-safeDeparse <- function(x, collapse=" ") paste(deparse(x, 500L), collapse=collapse)
+## R >= 4.0.0's deparse1() is a generalization
+if((Rv <- getRversion()) < "4.1.0") {
+  deparse1 <- function (expr, collapse = " ", width.cutoff = 500L, ...)
+      paste(deparse(expr, width.cutoff, ...), collapse = collapse)
+}
 
 abbrDeparse <- function(x, width=60) {
     r <- deparse(x, width)
     if(length(r) > 1) paste(r[1], "...") else r
 }
 
+sort_termlabs <- function(labs) {
+     if (length(labs)==0) return(labs)
+     ss <- strsplit(labs, ":")
+     ss <- sapply(ss, function(s) { if (length(s)==1) return(s); return(paste(sort(s),collapse=":")) })
+     return(sort(ss)) ## sort vector
+}
+
+## see whether mod1, mod2 are appropriate for Likelihood ratio testing
+CompareFixef <- function (mod1, mod2, component="cond") {
+     mr1 <- mod1$modelInfo$REML     
+     mr2 <- mod2$modelInfo$REML
+     if (mr1 != mr2) {
+        stop("Can't compare REML and ML fits", call.=FALSE)
+     }  
+     if (mr1 && mr2) {
+           tmpf <- function(obj) {   sort_termlabs(attr(terms(obj, component=component),"term.labels")) }
+           if (!identical(tmpf(mod1), tmpf(mod2))) {
+                stop("Can't compare REML fits with different fixed-effect components", call.=FALSE) 
+           }
+     }
+     return(TRUE) ## OK
+}
 
 ##' @importFrom methods is
 ##' @importFrom stats var getCall pchisq anova
@@ -897,17 +1050,21 @@ anova.glmmTMB <- function (object, ..., model.names = NULL)
 {
     mCall <- match.call(expand.dots = TRUE)
     dots <- list(...)
+    ## 'consistent' sapply, i.e. always unlist
     .sapply <- function(L, FUN, ...) unlist(lapply(L, FUN, ...))
     ## detect multiple models, i.e. models in ...
-    modp <- as.logical(vapply(dots, is, NA, "glmmTMB"))
+    modp <- as.logical(vapply(dots, FUN=is, "glmmTMB", FUN.VALUE=NA))
     if (any(modp)) {
         mods <- c(list(object), dots[modp])
         nobs.vec <- vapply(mods, nobs, 1L)
+        ## compare all models against first for being fitted consistently;
+        ## if all REML, fixed effects must be identical
+        vapply(mods[-1], CompareFixef, mod1=mods[[1]], FUN.VALUE=TRUE)
         if (var(nobs.vec) > 0)
             stop("models were not all fitted to the same size of dataset")
         if (is.null(mNms <- model.names))
             mNms <- vapply(as.list(mCall)[c(FALSE, TRUE, modp)],
-                           safeDeparse, "")
+                           deparse1, "")
         if (any(duplicated(mNms))) {
             warning("failed to find unique model names, assigning generic names")
             mNms <- paste0("MODEL", seq_along(mNms))
@@ -954,7 +1111,7 @@ anova.glmmTMB <- function (object, ..., model.names = NULL)
 #' @importFrom stats predict
 #' @export
 fitted.glmmTMB <- function(object, ...) {
-    predict(object,type="response")
+    predict(object,type="response", fast=TRUE)
 }
 
 .noSimFamilies <- NULL
@@ -1035,9 +1192,16 @@ formula.glmmTMB <- function(x, fixed.only=FALSE,
 
 ## need this so we can get contrasts carried through properly to model.matrix
 
+
+#' Methods for extracting developer-level information from \code{glmmTMB} models
+#' @rdname glmmTMB_methods
+#' @param object a fitted \code{glmmTMB} object
+#' @param component model component ("cond", "zi", or "disp"; not all models contain all components)
+#' @param part whether to return results for the fixed or random effect part of the model (at present only \code{part="fixed"} is implemented for most methods)
+#' @param \dots additional arguments (ignored or passed to \code{\link{model.frame}})
 #' @export
 
-model.matrix.glmmTMB <- function (object, ...)
+model.matrix.glmmTMB <- function (object, component="cond", part="fixed", ...)
 {
     ## FIXME: model.matrix.lm has this stuff -- what does it do/do we want it?
     ## if (n_match <- match("x", names(object), 0L))
@@ -1045,9 +1209,21 @@ model.matrix.glmmTMB <- function (object, ...)
     ## else {
     ## data <- model.frame(object, xlev = object$xlevels, ...)
 
-    data <- model.frame(object, ...)
-    NextMethod("model.matrix", data = data,
-               contrasts.arg = object$modelInfo$contrasts)
+    ## was calling NextMethod() on the model frame: failed after messing
+    ##   with terms structure
+    ## could be more efficient to extract $X, $Z rather than re-building
+    ## model matrix??
+    if (part != "fixed") stop("only fixed model matrices currently available")
+
+    ff <- object$modelInfo$allForm
+    form <- ff[[switch(component,
+                       cond="formula",
+                       zi="ziformula",
+                       disp="dispformula")]]
+    model.matrix(lme4::nobars(form), model.frame(object, ...),
+                 contrasts.arg = object$modelInfo$contrasts)
+    ## FIXME: what if contrasts are *different* for different components? (ugh)
+    ## should at least write a test to flag this case ...
 }
 
 ## convert ranef object to a long-format data frame, e.g. suitable
@@ -1058,10 +1234,8 @@ model.matrix.glmmTMB <- function (object, ...)
 ##' @export
 ##' @rdname ranef.glmmTMB
 ##' @param x a \code{ranef.glmmTMB} object (i.e., the result of running \code{ranef} on a fitted \code{glmmTMB} model)
-##' @param stringsAsFactors see \code{\link{data.frame}}
-as.data.frame.ranef.glmmTMB <- function(x,
-                ...,
-                stringsAsFactors = default.stringsAsFactors()) {
+as.data.frame.ranef.glmmTMB <- function(x, ...) {
+    check_dots(...)
     tmpf <- function(x) do.call(rbind,lapply(names(x),asDf0,x=x,id=TRUE))
     x0 <- lapply(x,tmpf)
     x1 <- Map(function(x,n) {
@@ -1085,14 +1259,16 @@ as.data.frame.ranef.glmmTMB <- function(x,
 #' @importFrom lme4 isLMM
 #' @importFrom lme4 refit
 ## don't export refit ...
-#' @description see \code{\link[lme4]{refit}} and \code{\link[lme4]{isLMM}} for details
+#' @description see \code{\link[lme4]{refit}} and \code{\link[lme4:isREML]{isLMM}} for details
 isLMM.glmmTMB <- function(object) {
    fam <- family(object)
    fam$family=="gaussian" && fam$link=="identity"
 }
 
 #' @export
+lme4::refit
 
+#' @export
 #' @rdname bootmer_methods
 #' @importFrom stats formula
 #' @param ... additional arguments (for generic consistency; ignored)
@@ -1103,6 +1279,10 @@ isLMM.glmmTMB <- function(object) {
 #'                   ziformula=~mined,
 #'                   data=Salamanders,
 #'                   family=nbinom1)
+#'    ## single parametric bootstrap step: refit with data simulated from original model
+#'    fm1R <- refit(fm1, simulate(fm1)[[1]])
+#'    ## the bootMer function from lme4 provides a wrapper for doing multiple refits
+#'    ##   with a specified summary function
 #'    b1 <- lme4::bootMer(fm1, FUN=function(x) fixef(x)$zi, nsim=20, .progress="txt")
 #'    if (requireNamespace("boot")) {
 #'       boot.ci(b1,type="perc")
@@ -1143,9 +1323,13 @@ refit.glmmTMB <- function(object, newresp, ...) {
           } else {
               stop("can't handle this data format, sorry ...")
           }
+      } else {
+          if (is.matrix(newresp)) newresp <- newresp[,1]
+          newdata[[deparse(fresp)]] <- newresp
       }
-  } else newdata[[deparse(fresp)]] <- newresp
-      
+  } else {
+      newdata[[deparse(fresp)]] <- newresp
+  }
   cc$data <- quote(newdata)
   return(eval(cc))
 }
@@ -1211,3 +1395,63 @@ coef.glmmTMB <- function(object,
     class(res) <- "coef.glmmTMB"
     return(res)
 }
+
+
+##' Extract weights from a glmmTMB object
+##'
+##' @details
+##' At present only explicitly specified
+##' \emph{prior weights} (i.e., weights specified
+##' in the \code{weights} argument) can be extracted from a fitted model.
+##' \itemize{
+##' \item Unlike other GLM-type models such as \code{\link{glm}} or
+##' \code{\link[lme4]{glmer}}, \code{weights()} does not currently return
+##' the total number of trials when binomial responses are specified
+##' as a two-column matrix.
+##' \item Since \code{glmmTMB} does not fit models via iteratively
+##' weighted least squares, \code{working weights} (see \code{\link[stats:glm]{weights.glm}}) are unavailable.
+##' }
+##' @importFrom stats model.frame
+##' @importFrom stats weights
+##' @param object a fitted \code{glmmTMB} object
+##' @param type weights type
+##' @param ... additional arguments (not used; for methods compatibility)
+##' @export
+weights.glmmTMB <- function(object, type="prior", ...) {
+    type <- match.arg(type)  ## other types are *not* OK
+    if (length(list(...)>0)) {
+        warning("unused arguments ignored: ",
+             paste(shQuote(names(list(...))),collapse=","))
+    }
+    stats::model.frame(object)[["(weights)"]]
+}
+
+# would like to export this only as a method, but not sure how ...
+# https://stackoverflow.com/questions/29079179/does-using-package-generics-require-the-package-to-be-in-depends-or-imports
+
+# extract model parameters
+#
+# This is a utility function for multcomp::glht
+# 
+## @param model fitted glmmTMB model
+## @param coef. function for retrieving coefficients
+## @param vcov. function for retrieving covariance matrix
+## @param df degrees of freedom
+## @param component which model component to test (cond, zi, or disp)
+
+##' @rawNamespace if(getRversion() >= "3.6.0") {
+##'      S3method(multcomp::modelparm, glmmTMB)
+##' } else {
+##'    export(modelparm.glmmTMB)
+##' }
+modelparm.glmmTMB <- function (model, coef. = function(x) fixef(x)[[component]],
+                               vcov. = function(x) vcov(x)[[component]],
+                               df = NULL, component="cond", ...) {
+    beta <- coef.(model)
+    sigma <- vcov.(model)
+    estimable <- unname(!is.na(beta))
+    if (is.null(df)) df <- 0
+    RET <- list(coef = beta, vcov = sigma, df = df, estimable = estimable)
+    class(RET) <- "modelparm"
+    return(RET)
+}
