@@ -22,8 +22,6 @@ matchForm <- function(obj, objU, family=FALSE) {
   return(objU)
 }
 
-context("Very basic glmmTMB fitting")
-
 lm0 <- lm(Reaction~Days,sleepstudy)
 fm00 <- glmmTMB(Reaction ~ Days, sleepstudy)
 fm0 <- glmmTMB(Reaction ~ 1    + ( 1  | Subject), sleepstudy)
@@ -104,8 +102,8 @@ test_that("Multiple RE, reordering", {
                     data = cbpp, family=binomial())
     tmb2 <- glmmTMB(cbind(incidence, size-incidence) ~ period + (1|obs) + (1|herd),
                     data = cbpp, family=binomial())
-    expect_equal(fixef(tmb1), fixef(tmb2),                   tolerance = 1e-13)
-    expect_equal(getME(tmb1, "theta"), getME(tmb2, "theta"), tolerance = 1e-13)
+    expect_equal(fixef(tmb1), fixef(tmb2),                   tolerance = 1e-8)
+    expect_equal(getME(tmb1, "theta"), getME(tmb2, "theta")[c(2,1)], tolerance = 5e-7)
 })
 
 test_that("Alternative family specifications [via update(.)]", {
@@ -239,10 +237,42 @@ test_that("NA handling", {
                  tolerance=1e-6)
 })
 
-quine.nb1 <- MASS::glm.nb(Days ~ Sex/(Age + Eth*Lrn), data = quine)
-quine.nb2 <- glmmTMB(Days ~ Sex/(Age + Eth*Lrn), data = quine,
-                     family=nbinom2())
-quine.nb3 <- glmmTMB(Days ~ Sex + (1|Age), data = quine,
-                     family=nbinom2())
+test_that("quine NB fit", {
+    quine.nb1 <- MASS::glm.nb(Days ~ Sex/(Age + Eth*Lrn), data = quine)
+    quine.nb2 <- glmmTMB(Days ~ Sex/(Age + Eth*Lrn), data = quine,
+                         family=nbinom2())
+    expect_equal(coef(quine.nb1),fixef(quine.nb2)[["cond"]],
+                 tolerance=1e-4)
+})
+## quine.nb3 <- glmmTMB(Days ~ Sex + (1|Age), data = quine,
+##                     family=nbinom2())
 
-## FIX ME: need to actually test these things!
+test_that("contrasts arg", {
+    quine.nb1 <- MASS::glm.nb(Days ~ Sex*Age, data = quine,
+                              contrasts=list(Sex="contr.sum",Age="contr.sum"))
+    quine.nb2 <- glmmTMB(Days ~ Sex*Age, data = quine,
+                         family=nbinom2(),
+                         contrasts=list(Sex="contr.sum",Age="contr.sum"))
+    expect_equal(coef(quine.nb1),fixef(quine.nb2)[["cond"]],
+                 tolerance=1e-4)
+})
+
+test_that("zero disp setting", {
+    skip_on_cran()
+    set.seed(101)
+    dd <- data.frame(y=rnorm(100),obs=1:100)
+    m0 <- glmmTMB(y~1, data=dd)
+    v0 <- sigma(m0)^2
+    m1 <- glmmTMB(y~1+(1|obs), data=dd)
+    tmpf <- function(x) c(sigma(x)^2,c(VarCorr(x)[["cond"]]$obs))
+    m <- -log10(sqrt(.Machine$double.eps))
+    pvec <- c(1,5,m,2*m,20)
+    res <- matrix(NA,ncol=2,nrow=length(pvec))
+    for (i in (seq_along(pvec))) {
+        mz <- update(m1,dispformula=~0,
+                     control=glmmTMBControl(zerodisp_val=log(10^(-pvec[i]))))
+        res[i,] <- tmpf(mz)
+    }
+    res <- rbind(res,tmpf(m1))
+    expect_true(var(res[,1]+res[,2])<1e-8)
+})
