@@ -1,8 +1,6 @@
 stopifnot(require("testthat"),
           require("glmmTMB"))
 
-context("glmmTMBControl")
-
 ## Some selected L1-distances between two fits
 distFits <- function(fit1, fit2) {
     s1 <- summary(fit1)
@@ -43,3 +41,39 @@ test_that("profile method", {
 
     expect_true( all( distFits(m1, m2) < c(1e-4, 1e-2, 1e-4) ) )
 })
+
+
+
+
+test_that("parallel regions", {
+
+
+  myfit <- function(...) {
+    glmmTMB(count ~ mined * spp + (1|site),
+            family = poisson,
+            data = Salamanders,
+            verbose = FALSE,
+            control = glmmTMBControl(...))
+  }
+
+  # Record time and model
+  capture_time_model <- function(...) {
+    start_time <- Sys.time()
+    model <- myfit(...)
+    end_time <- Sys.time()
+    return(list(model = model,
+                elapsed_time = end_time - start_time  ))
+  }
+
+
+  m1 <- capture_time_model( parallel = 1 )
+  ## DON'T grab all cores - bad on large machines
+  ## FIXME: check if parallel setting is persistent ???
+  m2 <- capture_time_model( parallel = min(4, parallel::detectCores()  ))
+
+  expect_true( all( distFits(m1[[1]], m2[[1]]) < c(1e-4, 1e-2, 1e-4) ) )
+
+  # expect_true( m1[[2]] <= m2[[2]])
+
+
+})
